/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mouse Gestures for Mozilla.
 *
 * The Initial Developer of the Original Code is Jens Bannmann.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Jens Bannmann <jens.b@web.de>
 *  Jochen <autohide@krickelkrackel.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var treeDS;
var mappingsTree;
var typesTree;
var tChildren;
var dmTree;
var mappingsContainer;
var defContainer;
var curType;
var curTypeLabel;
var typePrefix = new Array();
var mgBundle;

var mCodeCol;
var dmCodeCol;

function initMappingsWindow() {
  // get all elements we'll work with
  mgBundle = document.getElementById("mozgestbundle");
  mappingsTree = document.getElementById("mappingstree");
  typesTree = document.getElementById("typesTree");
  dmTree = document.getElementById("defaultmappingstree");
  mCodeCol = document.getElementById("codeCol");
  dmCodeCol = document.getElementById("dmCodeCol");

  initDirectionCodes();

  treeDS = new mgMappingsDS();
  mappingsTree.database.AddDataSource(treeDS);

  tChildren = document.getElementById("typestreechildren");
  var dsTypes = mgStorage.RDF.GetDataSourceBlocking("chrome://mozgest/content/windowTypes.rdf");
  var typesList = Components.classes['@mozilla.org/rdf/container;1']
                     .createInstance()
                     .QueryInterface(Components.interfaces.nsIRDFContainer);
  try {
    typesList.Init(dsTypes, mgStorage.RDF.GetResource("urn:mozgest:windows:list"));
  } catch (e) {
    dump("MozGest: Could not init typesList. " + e + "\n");
  }
  var typesEnum = typesList.GetElements();
  while (typesEnum.hasMoreElements()) {
    var type = typesEnum.getNext()
               .QueryInterface(Components.interfaces.nsIRDFResource);
    if (!mgStorage.getProperty(dsTypes, type, "separator")) {
      var typeCode = mgStorage.getProperty(dsTypes, type, "wtype");
      addTreeRow(tChildren,
                 new Array(mgStorage.getProperty(dsTypes, type, "label"),
                           typeCode));
      typePrefix[typeCode] = mgStorage.getProperty(dsTypes, type, "prefix");
    } else {
      tChildren.appendChild(document.createElement("treeseparator"));
    }
  }

  // this is called when window is opened for direct editing
  // (Sidebar and bookmark keywords for now)
  if (("arguments" in window)) {

      if (window.arguments[0] == "directEdit") {
        var ynode = typesTree.getElementsByAttribute("label",
                              window.arguments[2])[0].parentNode.parentNode;
        typesTree.contentView.selection.select(typesTree.builderView.getIndexOfItem(ynode));

        showMappingsFor(window.arguments[2]);
        setTimeout(handleArguments, 0, window.arguments[1]);
      }
  }
  else {
     typesTree.contentView.selection.select(0);
     typesTree.focus();
     showMappingsFor("window");
  }
  typesTree.addEventListener("select", showSelectedComponent, false);

  mCodeCol.addEventListener("DOMAttrModified", codeColChanged, true);
}

function handleArguments(xCompare) {

   // check for (custom)code (window.arguments[1])
   for (var y=0; y < mappingsTree.builderView.rowCount; y++)
   {   var ynode = mappingsTree.builderView.getResourceAtIndex(y);
       var yCompare=mgStorage.getProperty(treeDS, ynode, "code");
       var zCompare=mgStorage.getProperty(treeDS, ynode, "custom");

       if ((yCompare == xCompare) || (zCompare == xCompare)) {
          mappingsTree.builderView.selection.select(y);
          mappingsTree.boxObject.ensureRowIsVisible(y)
          mappingsTree.focus();
          editMapping(true);
          return;
          break;
       }
   }
   // no code/keyword: new gesture
   openEditMappingWindow("New", "", "", "", "", "", window.arguments[1]);
}

// synchronize column widths
function codeColChanged(event) {
  if (event.target == mCodeCol && event.attrName == "width") {
    dmCodeCol.setAttribute("width", event.newValue);
  }
}

function showSelectedComponent() {
  var row = typesTree.contentView.selection.currentIndex;
  if (!typesTree.treeBoxObject.view.isSeparator(row)) {
    if ("columns" in typesTree) {
       showMappingsFor(typesTree.view.getCellText(row, typesTree.columns["typeCol"]));
    }
    else {
       showMappingsFor(typesTree.treeBoxObject.view.getCellText(row, "typeCol"));
    }
  }
}

function showMappingsFor(component) {
  mappingsTree.ref = "urn:mozgest:mappings:" + component;
  mappingsTree.builder.rebuild();

  var dsDefault = mgStorage.getDefaultMappings();
  mappingsContainer = mgStorage.newContainer();
  defContainer = mgStorage.newContainer();
  var containerNode = mgStorage.RDF.GetResource(mappingsTree.ref);
  try {
    mappingsContainer.Init(treeDS, containerNode);
  } catch (e) {
    // create mappings container in treeDS...
    mgStorage.RDFCUtils.MakeSeq(treeDS, containerNode);
    mappingsContainer.Init(treeDS, containerNode);
    // ... and in user store to make sure treeDS can copy mappings back
    mgStorage.RDFCUtils.MakeSeq(mgStorage.getUserRDF(), containerNode);
    mgStorage.newContainer().Init(mgStorage.getUserRDF(), containerNode);
  }
  try {
    defContainer.Init(dsDefault, containerNode);
  } catch (e) {
    alert("MozGest: Fatal error - could not init defaultMappingsContainer.\n"
          + e);
  }
  curType = component;
  curTypeLabel = typesTree.contentView.getItemAtIndex(typesTree.currentIndex)
                 .firstChild.firstChild.getAttribute("label");
  showRemovedDefaults();

  //curType = component;
}

function showRemovedDefaults() {
  var dsDefault = mgStorage.getDefaultMappings();
  var mCode;
  var mFunction;
  var mapping;

  // determine which mappings are (will be) installed
  var installed = new Array();
  var mappingsEnum = mappingsContainer.GetElements();
  while (mappingsEnum.hasMoreElements()) {
    mapping = mappingsEnum.getNext();
    mCode = mgStorage.getProperty(treeDS, mapping, "code");
    mFunction = mgStorage.getProperty(treeDS, mapping, "function");

    // default mappings have non-localized codes, so we must deLocalize here
    installed[mgMappingLocalizer.deLocalize(mCode) + "~" + mFunction] = true;
  }

  // clear default mapping tree
  tChildren = document.getElementById("defMappingRows");
  while(tChildren.firstChild) {
    tChildren.removeChild(tChildren.firstChild);
  }

  // determine default mappings that will be missing & display them
  var defaultsEnum = defContainer.GetElements();
  while (defaultsEnum.hasMoreElements()) {
    mapping = defaultsEnum.getNext()
                  .QueryInterface(Components.interfaces.nsIRDFResource);
    mCode = mgStorage.getProperty(dsDefault, mapping, "code");
    mFunction = mgStorage.getProperty(dsDefault, mapping, "function");
    if (!((mCode + "~" + mFunction) in installed)) {
      var funcName = mgStorage.getProperty(dsDefault, mapping, "function");
      addTreeRow(tChildren, new Array(mgMappingLocalizer.localize(mCode),
                                  mgBundle.getString(funcName),
                                  mapping.Value));
    }
  }
}

function onAcceptMappings() {
  treeDS.applyChanges();
  return true;
}


/*** mapping actions ***/

function newMapping() {
  openEditMappingWindow("New", "", "", "", "", "", "");
}

function editMapping(passURI) {
  var row = mappingsTree.currentIndex;
  if (row < 0) {
    return;
  }
  var node = mappingsTree.builderView.getResourceAtIndex(row);
  openEditMappingWindow("Edit",
                        (passURI) ? node.Value : "",
                        mgStorage.getProperty(treeDS, node, "code"),
                        mgStorage.getProperty(treeDS, node, "appearance"),
                        mgStorage.getProperty(treeDS, node, "function"),
                        mgStorage.getProperty(treeDS, node, "name"),
                        decodeURI(mgStorage.getProperty(treeDS, node, "custom")));
}

function openEditMappingWindow(mode, uri, code, appearance, func, name, custom) {
  window.openDialog("chrome://mozgest/content/pref/edit-mapping.xul",
                    "mozgest:editmapping", "chrome,centerscreen,modal=yes",
                    mode, uri, code, appearance, func, name, custom,
                    typePrefix[curType], curType, curTypeLabel);
}

function receiveEditedMapping(uri, code, appearance,
                              func, name, custom, aWindowType) {
  if (!mgStorage.duplicateCheck(code, mappingsContainer, uri, treeDS)) {
    return false;
  }
  if (uri != "") {
    mgStorage.deleteNode(mgStorage.RDF.GetResource(uri), treeDS);
  }

  var node = mgStorage.RDF.GetAnonymousResource();
  mgStorage.saveMapping(treeDS, mappingsContainer, node,
                        code, appearance, func,
                        encodeURI(custom), name);

  showRemovedDefaults();
  return true;
}

function removeMapping() {
  var row = mappingsTree.currentIndex;
  if (row < 0) {
    return;
  }
  mgStorage.deleteNode(mappingsTree.builderView.getResourceAtIndex(row),
                       treeDS);
  showRemovedDefaults();
}

function publishMapping() {
  var row = mappingsTree.currentIndex;
  if (row < 0) {
    return;
  }
  var node = mappingsTree.builderView.getResourceAtIndex(row);
  window.openDialog("chrome://mozgest/content/pref/publishMapping.xul",
                    "mozgest:publishmapping", "chrome,centerscreen,modal=yes",
                    curType,
                    mgStorage.getProperty(treeDS, node, "code"),
                    mgStorage.getProperty(treeDS, node, "appearance"),
                    mgStorage.getProperty(treeDS, node, "function"),
                    mgStorage.getProperty(treeDS, node, "name"),
                    decodeURI(mgStorage.getProperty(treeDS, node, "custom")));
}


/*** commands for default mappings ***/

function addDefault() {
  // determine RDF node (get URN from dmResCol column)
  var row = dmTree.contentView.selection.currentIndex;
  if (row < 0) {
    return false;
  }

  var locCode;
  if ("columns" in dmTree) {
     locCode = dmTree.view.getCellText(row, dmTree.columns["dmCodeCol"]);
  }
  else {
      locCode = dmTree.treeBoxObject.view.getCellText(row,"dmCodeCol");
  }

  if (!mgStorage.duplicateCheck(locCode, mappingsContainer, "", treeDS)) {
    return false;
  }

  var urn;
  if ("columns" in dmTree) {
     urn = dmTree.view.getCellText(row, dmTree.columns["dmResCol"]);
  }
  else {
     urn = dmTree.treeBoxObject.view.getCellText(row, "dmResCol");
  }

  var sourceNode = mgStorage.RDF.GetResource(urn);

  // read the mapping's data
  var dsDefault = mgStorage.getDefaultMappings();
  var mFunction = mgStorage.getProperty(dsDefault, sourceNode, "function");
  var mAppearance = mgStorage.getProperty(dsDefault, sourceNode, "appearance");

  // create new node for treeDS (with localized data)
  var newNode = mgStorage.RDF.GetAnonymousResource();
  mgStorage.saveMapping(treeDS, mappingsContainer, newNode,
                        locCode, mgMappingLocalizer.localize(mAppearance),
                        mFunction, "", mgMappingLocalizer.getString(mFunction));
  showRemovedDefaults();
  return true;
}